@@ -1,7 +1,7 @@
 /* -*- C++ -*- */
 /*
    Copyright (c) 2002, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2020, MariaDB
+   Copyright (c) 2020, 2022, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,6 +47,14 @@ class sp_instr;
 class sp_instr_opt_meta;
 class sp_instr_jump_if_not;
 
+/**
+  Number of PSI_statement_info instruments
+  for internal stored programs statements.
+*/
+#ifdef HAVE_PSI_INTERFACE
+void init_sp_psi_keys(void);
+#endif
+
 /*************************************************************************/
 
 /**
@@ -174,6 +182,11 @@ public:
   const Sp_handler *m_handler;
   uint m_flags;                 // Boolean attributes of a stored routine
 
+  /**
+    Instrumentation interface for SP.
+  */
+  PSI_sp_share *m_sp_share;
+
   Column_definition m_return_field_def; /**< This is used for FUNCTIONs only. */
 
   const char *m_tmp_query;	///< Temporary pointer to sub query string
@@ -548,7 +561,7 @@ public:
   { return m_flags & MODIFIES_DATA; }
 
   inline uint instructions()
-  { return m_instr.elements; }
+  { return (uint)m_instr.elements; }
 
   inline sp_instr *
   last_instruction()
@@ -873,6 +886,8 @@ public:
     return NULL;
   }
 
+  virtual void init_psi_share();
+
 protected:
 
   MEM_ROOT *m_thd_root;		///< Temp. store for thd's mem_root
@@ -948,9 +963,9 @@ public:
   public:
     LexList() { elements= 0; }
     // Find a package routine by a non qualified name
-    LEX *find(const LEX_CSTRING &name, stored_procedure_type type);
+    LEX *find(const LEX_CSTRING &name, enum_sp_type type);
     // Find a package routine by a package-qualified name, e.g. 'pkg.proc'
-    LEX *find_qualified(const LEX_CSTRING &name, stored_procedure_type type);
+    LEX *find_qualified(const LEX_CSTRING &name, enum_sp_type type);
     // Check if a routine with the given qualified name already exists
     bool check_dup_qualified(const LEX_CSTRING &name, const Sp_handler *sph)
     {
@@ -1007,6 +1022,7 @@ public:
            m_routine_implementations.push_back(lex, &main_mem_root);
   }
   sp_package *get_package() { return this; }
+  void init_psi_share();
   bool is_invoked() const
   {
     /*
@@ -1042,8 +1058,9 @@ public:
     Query_arena(thd->lex->sphead->get_main_mem_root(), STMT_INITIALIZED_FOR_SP)
   { }
   ~sp_lex_cursor() { free_items(); }
-  void cleanup_stmt() { }
-  Query_arena *query_arena() { return this; }
+  virtual bool cleanup_stmt(bool /*restore_set_statement_vars*/) override
+  { return false; }
+  Query_arena *query_arena() override { return this; }
   bool validate()
   {
     DBUG_ASSERT(sql_command == SQLCOM_SELECT);
@@ -1173,6 +1190,7 @@ public:
   {
     m_ip= dst;
   }
+  virtual PSI_statement_info* get_psi_info() = 0;
 
 }; // class sp_instr : public Sql_alloc
 
@@ -1299,6 +1317,10 @@ private:
 
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
+
 }; // class sp_instr_stmt : public sp_instr
 
 
@@ -1333,6 +1355,10 @@ protected:
   uint m_offset;		///< Frame offset
   Item *m_value;
   sp_lex_keeper m_lex_keeper;
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_set : public sp_instr
 
 
@@ -1441,6 +1467,10 @@ private:
   Item_trigger_field *trigger_field;
   Item *value;
   sp_lex_keeper m_lex_keeper;
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_trigger_field : public sp_instr
 
 
@@ -1527,6 +1557,9 @@ public:
       m_dest= new_dest;
   }
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_jump : public sp_instr_opt_meta
 
 
@@ -1578,6 +1611,9 @@ private:
   Item *m_expr;			///< The condition
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_jump_if_not : public sp_instr_jump
 
 
@@ -1595,17 +1631,9 @@ public:
   virtual ~sp_instr_preturn()
   {}
 
-  virtual int execute(THD *thd, uint *nextp)
-  {
-    DBUG_ENTER("sp_instr_preturn::execute");
-    *nextp= UINT_MAX;
-    DBUG_RETURN(0);
-  }
+  virtual int execute(THD *thd, uint *nextp);
 
-  virtual void print(String *str)
-  {
-    str->append(STRING_WITH_LEN("preturn"));
-  }
+  virtual void print(String *str);
 
   virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads)
   {
@@ -1613,6 +1641,9 @@ public:
     return UINT_MAX;
   }
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_preturn : public sp_instr
 
 
@@ -1650,6 +1681,9 @@ protected:
   const Type_handler *m_type_handler;
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_freturn : public sp_instr
 
 
@@ -1715,6 +1749,9 @@ private:
   // debug version only). It's used in print().
   uint m_frame;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_hpush_jump : public sp_instr_jump
 
 
@@ -1745,6 +1782,9 @@ private:
 
   uint m_count;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_hpop : public sp_instr
 
 
@@ -1779,12 +1819,14 @@ private:
 
   uint m_frame;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_hreturn : public sp_instr_jump
 
 
 /** This is DECLARE CURSOR */
-class sp_instr_cpush : public sp_instr,
-                       public sp_cursor
+class sp_instr_cpush : public sp_instr, public sp_cursor
 {
   sp_instr_cpush(const sp_instr_cpush &); /**< Prevent use of these */
   void operator=(sp_instr_cpush &);
@@ -1798,21 +1840,25 @@ public:
   virtual ~sp_instr_cpush()
   {}
 
-  virtual int execute(THD *thd, uint *nextp);
+  int execute(THD *thd, uint *nextp) override;
 
-  virtual void print(String *str);
+  void print(String *str) override;
 
   /**
     This call is used to cleanup the instruction when a sensitive
     cursor is closed. For now stored procedures always use materialized
     cursors and the call is not used.
   */
-  virtual void cleanup_stmt() { /* no op */ }
+  virtual bool cleanup_stmt(bool /*restore_set_statement_vars*/) override
+  { return false; }
 private:
 
   sp_lex_keeper m_lex_keeper;
   uint m_cursor;                /**< Frame offset (for debugging) */
 
+public:
+  PSI_statement_info* get_psi_info() override { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cpush : public sp_instr
 
 
@@ -1843,6 +1889,9 @@ private:
 
   uint m_count;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cpop : public sp_instr
 
 
@@ -1870,6 +1919,9 @@ private:
 
   uint m_cursor;		///< Stack index
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_copen : public sp_instr_stmt
 
 
@@ -1897,6 +1949,10 @@ public:
   virtual int execute(THD *thd, uint *nextp);
   virtual int exec_core(THD *thd, uint *nextp);
   virtual void print(String *str);
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 };
 
 
@@ -1922,6 +1978,9 @@ private:
 
   uint m_cursor;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cclose : public sp_instr
 
 
@@ -1956,6 +2015,9 @@ private:
   List<sp_variable> m_varlist;
   bool m_error_on_no_data;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cfetch : public sp_instr
 
 /*
@@ -1980,6 +2042,10 @@ public:
   virtual int execute(THD *thd, uint *nextp);
 
   virtual void print(String *str);
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_agg_cfetch : public sp_instr
 
 
@@ -2013,6 +2079,9 @@ private:
 
   int m_errcode;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_error : public sp_instr
 
 
@@ -2052,6 +2121,9 @@ private:
   Item *m_case_expr;
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_set_case_expr : public sp_instr_opt_meta
 
 bool check_show_routine_access(THD *thd, sp_head *sp, bool *full_access);
