@@ -306,6 +306,9 @@ protected:
   /** SQL CURSOR_NAME condition item. */
   String m_cursor_name;
 
+  /** SQL ROW_NUMBER condition item. */
+  ulong m_row_number;
+
   Sql_condition_items()
    :m_class_origin((const char*) NULL, 0, & my_charset_utf8mb3_bin),
     m_subclass_origin((const char*) NULL, 0, & my_charset_utf8mb3_bin),
@@ -316,7 +319,8 @@ protected:
     m_schema_name((const char*) NULL, 0, & my_charset_utf8mb3_bin),
     m_table_name((const char*) NULL, 0, & my_charset_utf8mb3_bin),
     m_column_name((const char*) NULL, 0, & my_charset_utf8mb3_bin),
-    m_cursor_name((const char*) NULL, 0, & my_charset_utf8mb3_bin)
+    m_cursor_name((const char*) NULL, 0, & my_charset_utf8mb3_bin),
+    m_row_number(0)
   { }
 
   void clear()
@@ -331,6 +335,7 @@ protected:
     m_table_name.length(0);
     m_column_name.length(0);
     m_cursor_name.length(0);
+    m_row_number= 0;
   }
 };
 
@@ -434,16 +439,14 @@ private:
     @param level    - the error level for this condition
     @param msg      - the message text for this condition
   */
-  Sql_condition(MEM_ROOT *mem_root,
-                const Sql_condition_identity &value,
-                const char *msg)
-   :Sql_condition_identity(value),
-    m_mem_root(mem_root)
+  Sql_condition(MEM_ROOT *mem_root, const Sql_condition_identity &value,
+                const char *msg, ulong current_row_for_warning)
+   : Sql_condition_identity(value), m_mem_root(mem_root)
   {
-    DBUG_ASSERT(mem_root != NULL);
     DBUG_ASSERT(value.get_sql_errno() != 0);
     DBUG_ASSERT(msg != NULL);
     set_builtin_message_text(msg);
+    m_row_number= current_row_for_warning;
   }
 
   /** Destructor. */
@@ -720,7 +723,7 @@ private:
   void inc_current_row_for_warning() { m_current_row_for_warning++; }
 
   /** Reset the current row counter. Start counting from the first row. */
-  void reset_current_row_for_warning() { m_current_row_for_warning= 1; }
+  void reset_current_row_for_warning(int n) { m_current_row_for_warning= n; }
 
   /** Return the current counter value. */
   ulong current_row_for_warning() const { return m_current_row_for_warning; }
@@ -742,9 +745,8 @@ private:
 
     @return a pointer to the added SQL-condition.
   */
-  Sql_condition *push_warning(THD *thd,
-                              const Sql_condition_identity *identity,
-                              const char* msg);
+  Sql_condition *push_warning(THD *thd, const Sql_condition_identity *identity,
+                              const char* msg, ulong current_row_number);
 
   /**
     Add a new SQL-condition to the current list and increment the respective
@@ -811,8 +813,8 @@ private:
 };
 
 
-extern char *err_conv(char *buff, uint to_length, const char *from,
-                      uint from_length, CHARSET_INFO *from_cs);
+extern size_t err_conv(char *buff, uint to_length, const char *from,
+                       uint from_length, CHARSET_INFO *from_cs);
 
 class ErrBuff
 {
@@ -824,31 +826,37 @@ public:
     err_buffer[0]= '\0';
   }
   const char *ptr() const { return err_buffer; }
-  const char *set_longlong(const Longlong_hybrid &nr) const
+  LEX_CSTRING set_longlong(const Longlong_hybrid &nr) const
   {
-    return nr.is_unsigned() ? ullstr(nr.value(), err_buffer) :
-                              llstr(nr.value(), err_buffer);
+    int radix= nr.is_unsigned() ? 10 : -10;
+    const char *end= longlong10_to_str(nr.value(), err_buffer, radix);
+    DBUG_ASSERT(end >= err_buffer);
+    return {err_buffer, (size_t) (end - err_buffer)};
   }
-  const char *set_double(double nr) const
+  LEX_CSTRING set_double(double nr) const
   {
-    my_gcvt(nr, MY_GCVT_ARG_DOUBLE, sizeof(err_buffer), err_buffer, 0);
-    return err_buffer;
+    size_t length= my_gcvt(nr, MY_GCVT_ARG_DOUBLE,
+                           sizeof(err_buffer), err_buffer, 0);
+    return {err_buffer, length};
   }
-  const char *set_decimal(const decimal_t *d) const
+  LEX_CSTRING set_decimal(const decimal_t *d) const
   {
-    int len= sizeof(err_buffer);
-    decimal2string(d, err_buffer, &len, 0, 0, ' ');
-    return err_buffer;
+    int length= sizeof(err_buffer);
+    decimal2string(d, err_buffer, &length, 0, 0, ' ');
+    DBUG_ASSERT(length >= 0);
+    return {err_buffer, (size_t) length};
   }
-  const char *set_str(const char *str, size_t len, CHARSET_INFO *cs) const
+  LEX_CSTRING set_str(const char *str, size_t len, CHARSET_INFO *cs) const
   {
     DBUG_ASSERT(len < UINT_MAX32);
-    return err_conv(err_buffer, (uint) sizeof(err_buffer), str, (uint) len, cs);
+    len= err_conv(err_buffer, (uint) sizeof(err_buffer), str, (uint) len, cs);
+    return {err_buffer, len};
   }
-  const char *set_mysql_time(const MYSQL_TIME *ltime) const
+  LEX_CSTRING set_mysql_time(const MYSQL_TIME *ltime) const
   {
-    my_TIME_to_str(ltime, err_buffer, AUTO_SEC_PART_DIGITS);
-    return err_buffer;
+    int length= my_TIME_to_str(ltime, err_buffer, AUTO_SEC_PART_DIGITS);
+    DBUG_ASSERT(length >= 0);
+    return {err_buffer, (size_t) length};
   }
 };
 
@@ -858,7 +866,11 @@ class ErrConv: public ErrBuff
 public:
   ErrConv() {}
   virtual ~ErrConv() {}
-  virtual const char *ptr() const = 0;
+  virtual LEX_CSTRING lex_cstring() const= 0;
+  inline const char *ptr() const
+  {
+    return lex_cstring().str;
+  }
 };
 
 class ErrConvString : public ErrConv
@@ -873,7 +885,7 @@ public:
     : ErrConv(), str(str_arg), len(strlen(str_arg)), cs(cs_arg) {}
   ErrConvString(const String *s)
     : ErrConv(), str(s->ptr()), len(s->length()), cs(s->charset()) {}
-  const char *ptr() const
+  LEX_CSTRING lex_cstring() const override
   {
     return set_str(str, len, cs);
   }
@@ -884,7 +896,7 @@ class ErrConvInteger : public ErrConv, public Longlong_hybrid
 public:
   ErrConvInteger(const Longlong_hybrid &nr)
    : ErrConv(), Longlong_hybrid(nr) { }
-  const char *ptr() const
+  LEX_CSTRING lex_cstring() const override
   {
     return set_longlong(static_cast<Longlong_hybrid>(*this));
   }
@@ -895,7 +907,7 @@ class ErrConvDouble: public ErrConv
   double num;
 public:
   ErrConvDouble(double num_arg) : ErrConv(), num(num_arg) {}
-  const char *ptr() const
+  LEX_CSTRING lex_cstring() const override
   {
     return set_double(num);
   }
@@ -906,7 +918,7 @@ class ErrConvTime : public ErrConv
   const MYSQL_TIME *ltime;
 public:
   ErrConvTime(const MYSQL_TIME *ltime_arg) : ErrConv(), ltime(ltime_arg) {}
-  const char *ptr() const
+  LEX_CSTRING lex_cstring() const override
   {
     return set_mysql_time(ltime);
   }
@@ -917,7 +929,7 @@ class ErrConvDecimal : public ErrConv
   const decimal_t *d;
 public:
   ErrConvDecimal(const decimal_t *d_arg) : ErrConv(), d(d_arg) {}
-  const char *ptr() const
+  LEX_CSTRING lex_cstring() const override
   {
     return set_decimal(d);
   }
@@ -960,6 +972,8 @@ public:
     DA_EOF,
     /** Set whenever one calls my_ok() in PS bulk mode. */
     DA_OK_BULK,
+    /** Set whenever one calls my_eof() in PS bulk mode. */
+    DA_EOF_BULK,
     /** Set whenever one calls my_error() or my_message(). */
     DA_ERROR,
     /** Set in case of a custom response, such as one from COM_STMT_PREPARE. */
@@ -1019,8 +1033,11 @@ public:
   enum_diagnostics_status status() const { return m_status; }
 
   const char *message() const
-  { DBUG_ASSERT(m_status == DA_ERROR || m_status == DA_OK ||
-                m_status == DA_OK_BULK); return m_message; }
+  {
+    DBUG_ASSERT(m_status == DA_ERROR || m_status == DA_OK ||
+                m_status == DA_OK_BULK || m_status == DA_EOF_BULK);
+    return m_message;
+  }
 
 
   uint sql_errno() const
@@ -1047,7 +1064,7 @@ public:
   uint statement_warn_count() const
   {
     DBUG_ASSERT(m_status == DA_OK || m_status == DA_OK_BULK ||
-                m_status == DA_EOF);
+                m_status == DA_EOF ||m_status == DA_EOF_BULK );
     return m_statement_warn_count;
   }
 
@@ -1128,8 +1145,8 @@ public:
   void inc_current_row_for_warning()
   { get_warning_info()->inc_current_row_for_warning(); }
 
-  void reset_current_row_for_warning()
-  { get_warning_info()->reset_current_row_for_warning(); }
+  void reset_current_row_for_warning(int n)
+  { get_warning_info()->reset_current_row_for_warning(n); }
 
   bool is_warning_info_read_only() const
   { return get_warning_info()->is_read_only(); }
@@ -1160,10 +1177,12 @@ public:
                               const char* sqlstate,
                               Sql_condition::enum_warning_level level,
                               const Sql_user_condition_identity &ucid,
-                              const char* msg)
+                              const char* msg,
+                              ulong current_row_number)
   {
     Sql_condition_identity tmp(sql_errno_arg, sqlstate, level, ucid);
-    return get_warning_info()->push_warning(thd, &tmp, msg);
+    return get_warning_info()->push_warning(thd, &tmp, msg,
+                                            current_row_number);
   }
 
   Sql_condition *push_warning(THD *thd,
@@ -1173,7 +1192,7 @@ public:
                               const char* msg)
   {
     return push_warning(thd, sqlerrno, sqlstate, level,
-                        Sql_user_condition_identity(), msg);
+                        Sql_user_condition_identity(), msg, 0);
   }
   void mark_sql_conditions_for_removal()
   { get_warning_info()->mark_sql_conditions_for_removal(); }
